@@ -4,6 +4,7 @@ from hashlib import md5
 from aquests.protocols.http import http_util
 import base64
 from functools import wraps
+from skitai.exceptions import HTTPError
 
 class AuthorizedUser:
     def __init__ (self, user, realm, info = None):
@@ -154,6 +155,16 @@ class Auth:
             request.user = www_authenticate
         return True
 
+    # CSRF ------------------------------------------------------
+    def csrf_verification_required (self, f):
+        self.save_function_spec (f)
+        @wraps(f)
+        def wrapper (was, *args, **kwargs):
+            if not was.csrf_verify ():
+                raise was.Error ("400 Bad Request", 'CSRF token error')
+            return f (was, *args, **kwargs)
+        return wrapper
+
     # Auth ------------------------------------------------------
     def bearer_handler (self, f):
         self._decos ["bearer_handler"] = f
@@ -189,11 +200,28 @@ class Auth:
         self.set_auth_flag (f, ('login', None))
         @wraps(f)
         def wrapper (was, *args, **kwargs):
-            _funcs = self._decos.get ("login_handler")
-            if _funcs:
-                response = _funcs (was)
-                if response is not None:
-                    return response
+            try:
+                _funcs = self._decos ["login_handler"]
+            except KeyError:
+                raise RuntimeError ('login_handler is not defined')
+            response = _funcs (was)
+            if response is not None:
+                return response
+            return f (was, *args, **kwargs)
+        return wrapper
+
+    def clarify_login (self, f):
+        self.save_function_spec (f)
+        @wraps(f)
+        def wrapper (was, *args, **kwargs):
+            try:
+                _funcs = self._decos ["login_handler"]
+            except KeyError:
+                raise RuntimeError ('login_handler is not defined')
+            try:
+                _funcs (was)
+            except HTTPError:
+                pass
             return f (was, *args, **kwargs)
         return wrapper
 
@@ -207,11 +235,13 @@ class Auth:
         self.set_auth_flag (f, 'staff')
         @wraps(f)
         def wrapper (was, *args, **kwargs):
-            _funcs = self._decos.get ("staff_member_check_handler")
-            if _funcs:
-                response = _funcs (was)
-                if response is not None:
-                    return response
+            try:
+                _funcs = self._decos ["staff_member_check_handler"]
+            except KeyError:
+                raise RuntimeError ('staff_member_check_handler is not defined')
+            response = _funcs (was)
+            if response is not None:
+                return response
             return f (was, *args, **kwargs)
         return wrapper
 
@@ -220,6 +250,21 @@ class Auth:
         self._decos ["permission_check_handler"] = f
         return f
 
+    def clarify_permission (self, f):
+        self.save_function_spec (f)
+        @wraps(f)
+        def wrapper (was, *args, **kwargs):
+            try:
+                _funcs = self._decos ["permission_check_handler"]
+            except KeyError:
+                raise RuntimeError ('permission_check_handler is not defined')
+            try:
+                _funcs (was, [])
+            except HTTPError:
+                pass
+            return f (was, *args, **kwargs)
+        return wrapper
+
     METHODS = {"POST", "GET", "PUT", "DELETE", "PATCH"}
     def permission_required (self, __donotusethisvariable__ = None, **kargs):
         if not __donotusethisvariable__:
@@ -240,28 +285,31 @@ class Auth:
             self._permission_map [f] = (__donotusethisvariable__, methods, specified)
             @wraps(f)
             def wrapper (was, *args, **kwargs):
-                _funcs = self._decos.get ("permission_check_handler")
-                if _funcs:
-                    defaults, methods, specified = self._permission_map [f]
-                    perms = set ()
-                    for k, v in specified.items ():
-                        specific = was.request.ARGS.get (k)
-                        if specific == "notme" and was.request.method not in ("GET", "OPTIONS"):
-                            raise was.Error ("421 Method Not Allowed")
-                        if specific and specific != "me":
-                            perms = v.copy ()
-                            break
-                    perms2 = methods.get (was.request.method, {})
-                    if perms2:
-                        if perms:
-                            perms = perms.union (perms2)
-                        else:
-                            prems = perms2.copy ()
-                    if not perms:
-                        perms = defaults
-                    response = _funcs (was, perms)
-                    if response is not None:
-                        return response
+                try:
+                    _funcs = self._decos ["permission_check_handler"]
+                except KeyError:
+                    raise RuntimeError ('permission_check_handler is not defined')
+
+                defaults, methods, specified = self._permission_map [f]
+                perms = set ()
+                for k, v in specified.items ():
+                    specific = was.request.ARGS.get (k)
+                    if specific == "notme" and was.request.method not in ("GET", "OPTIONS"):
+                        raise was.Error ("421 Method Not Allowed")
+                    if specific and specific != "me":
+                        perms = v.copy ()
+                        break
+                perms2 = methods.get (was.request.method, {})
+                if perms2:
+                    if perms:
+                        perms = perms.union (perms2)
+                    else:
+                        prems = perms2.copy ()
+                if not perms:
+                    perms = defaults
+                response = _funcs (was, perms)
+                if response is not None:
+                    return response
                 return f (was, *args, **kwargs)
             return wrapper
         return decorator
