@@ -181,9 +181,9 @@ class WAS (wsgiappservice.WAS):
         return '<input type="hidden" name="{}" value="{}">'.format (self.CSRF_NAME, self.csrf_token)
 
     def csrf_verify (self, keep = False):
-        if not self.request.args.get (self.CSRF_NAME):
+        token = self.request.get_header ('x-csrf-token', self.request.args.get (self.CSRF_NAME))
+        if not token:
             return False
-        token = self.request.args [self.CSRF_NAME]
         if self.csrf_token == token:
             if not keep:
                 self.session.mount ()
