@@ -1,6 +1,6 @@
 from aquests.protocols.http import http_util
 from . import cookie
-from rs4.reraise import reraise 
+from rs4.reraise import reraise
 import sys
 import json
 from aquests.athreads import trigger
@@ -9,8 +9,12 @@ from aquests.protocols.http import respcodes
 from skitai.exceptions import HTTPError
 from skitai.wastuff.api import API
 import warnings
+try:
+	from css_html_js_minify import html_minify
+except ImportError:
+	html_minify = None
 
-def traceback ():	
+def traceback ():
 	t, v, tb = sys.exc_info ()
 	tbinfo = []
 	assert tb # Must have a traceback
@@ -26,42 +30,42 @@ def traceback ():
 	file, function, line = tbinfo [-1]
 	return (
 		"%s %s, file %s at line %s, %s" % (
-			t, v, file, line, 
+			t, v, file, line,
 			function == "?" and "__main__" or "function " + function
 		)
 	)
 
-		
+
 class Executor:
 	def __init__ (self, env, get_method):
-		self.env = env	
+		self.env = env
 		self.get_method = get_method
 		self.was = None
-	
+
 	def chained_exec (self, method, args, karg, make_list = True):
 		# recursive before, after, teardown
 		# [b, [b, [b, func, s, f, t], s, f, t], s, f, t]
-		
+
 		response = None
-		[before, func, success, failed, teardown] = method	
+		[before, func, success, failed, teardown] = method
 		try:
 			if before:
 				response = before (self.was)
 				if response is not None:
 					return response
-					
+
 			if type (func) is list:
 				response = self.chained_exec (func, args, karg, make_list)
-					
+
 			else:
 				response = func (self.was, *args, **karg)
 				if make_list and type (response) is not list:
-					response = [response]											
-				
+					response = [response]
+
 		except MemoryError:
 			raise
-																										
-		except Exception as expt:			
+
+		except Exception as expt:
 			exc_info = sys.exc_info ()
 			self.was.subapp.emit ("request:failed", exc_info)
 			if failed:
@@ -71,11 +75,11 @@ class Executor:
 			else:
 				# filed handle exception and contents, just log
 				self.was.traceback ()
-							
+
 		else:
 			self.was.subapp.emit ("request:success")
 			success and success (self.was)
-		
+
 		self.was.subapp.emit ("request:teardown")
 		teardown and teardown (self.was)
 		return response
@@ -95,35 +99,35 @@ class Executor:
 		if urlargs:
 			required = set (args [:urlargs])
 			if defaults:
-				required = required.difference (set (defaults.keys ()))			
+				required = required.difference (set (defaults.keys ()))
 			for arg in required:
 				if arg not in karg:
 					raise HTTPError ("400 Bad Request", "conflict url parameters")
 
 		required = set (args)
 		if defaults:
-			required = required.difference (set (defaults.keys ()))					
+			required = required.difference (set (defaults.keys ()))
 
 		for r in required:
 			if r not in karg:
 				raise HTTPError ("400 Bad Request", "parameter `{}` is missing".format (r))
 		if self.was.request.routable.get ("keywords"):
 			return
-		
+
 		for r in karg:
 			if r not in args:
-				raise HTTPError ("400 Bad Request", "parameter `{}` needn't".format (r))	
+				raise HTTPError ("400 Bad Request", "parameter `{}` needn't".format (r))
 
 	def generate_content (self, method, _args, karg):
-		karg = self.parse_kargs (karg)		
+		karg = self.parse_kargs (karg)
 		try:
-			response = self.chained_exec (method, _args, karg)			
-		except TypeError:			
+			response = self.chained_exec (method, _args, karg)
+		except TypeError:
 			self.was.traceback ()
 			self.verify_args_lazy (karg) #lazy validating request parameters for respond 400
 			raise
 		return response
-	
+
 	def is_calling_args_group (self, data, forward = True):
 		if self.was.request.routable.get ('keywords'):
 			return True
@@ -135,48 +139,51 @@ class Executor:
 				return True
 			for k in wanted_args:
 				if k in data:
-					return True		
-		else:		
+					return True
+		else:
 			for i in range (-1, -(len (wanted_args) + 1), -1):
 				if wanted_args [i] in data:
-					return True		
+					return True
 		return False
-				
+
 	def parse_kargs (self, kargs):
 		self.was.request.PARAMS = kargs.copy ()
 		query = self.env.get ("QUERY_STRING")
 		data = self.was.request.dict ()
-		
-		allkarg = AttrDict ()		
+
+		allkarg = AttrDict ()
 		self.merge_args (allkarg, kargs)
-		
+
 		if not query and not data:
 			self.was.request.set_args (allkarg)
 			return kargs
-		
+
 		query_included = True
 		if query:
-			querydict = http_util.crack_query (query)
+			try:
+				querydict = http_util.crack_query (query)
+			except IndexError:
+				raise HTTPError ("400 Error", 'invalid query string')
 			self.was.request.URL = querydict
 			self.merge_args (allkarg, querydict)
-			if self.is_calling_args_group (querydict):				
-				if not data:		
+			if self.is_calling_args_group (querydict):
+				if not data:
 					self.was.request.set_args (allkarg)
-					return allkarg				
+					return allkarg
 				self.merge_args (kargs, querydict)
 			else:
 				query_included = False
-		
+
 		if data:
 			self.merge_args (allkarg, data, overwrite = True)
 			if query_included and self.is_calling_args_group (data, False):
 				self.was.request.set_args (allkarg)
 				return allkarg
-			
+
 		self.was.request.set_args (allkarg)
 		return kargs
-		
-	def merge_args (self, s, n, overwrite = False):		
+
+	def merge_args (self, s, n, overwrite = False):
 		for k, v in list(n.items ()):
 			if k in s:
 				if overwrite:
@@ -185,9 +192,9 @@ class Executor:
 				if type (s [k]) is not list:
 					s [k] = [s [k]]
 				s [k].append (v)
-			else:	 
+			else:
 				s [k] = v
-	
+
 	def commit (self):
 		if self.was.app is None:
 			# this is failed request
@@ -196,61 +203,65 @@ class Executor:
 		if hasattr (self.was.request, "django"):
 			self.was.request.django.commit ()
 		if not self.was.in__dict__ ("cookie"):
-			return		
+			return
 		if self.was.in__dict__ ("session"):
 			self.was.session and self.was.session.commit ()
 		if self.was.in__dict__ ("mbox"):
-			self.was.mbox and self.was.mbox.commit ()			
-		self.was.cookie.commit ()		
-	
+			self.was.mbox and self.was.mbox.commit ()
+		self.was.cookie.commit ()
+
 	def rollback (self):
 		if self.was.app is None:
 			# this is failed request
-			return		
-		if not self.was.in__dict__ ("cookie"):		
-			return			
+			return
+		if not self.was.in__dict__ ("cookie"):
+			return
 		# keep commit order, session -> mbox -> cookie
-		if self.was.in__dict__ ("session"):		
+		if self.was.in__dict__ ("session"):
 			self.was.session and self.was.session.rollback ()
 		if self.was.in__dict__ ("mbox"):
 			self.was.mbox and self.was.mbox.rollback ()
 		self.was.cookie.rollback ()
-	
+
 	def find_method (self, request, path, handle_response = True):
 		current_app, thing, param, options, respcode = self.get_method (
-			path, 
+			path,
 			request
 		)
-		
-		if respcode and handle_response:			
+
+		if respcode and handle_response:
 			if respcode == 301:
 				request.response ["Location"] = thing
-				request.response.error (301, "Object Moved", why = 'Object Moved To <a href="%s">Here</a>' % thing)							
+				request.response.error (301, "Object Moved", why = 'Object Moved To <a href="%s">Here</a>' % thing)
 			elif respcode != 200:
 				request.response.error (respcode, respcodes.get (respcode, "Undefined Error"))
 
 		if thing:
-			request.salt = current_app.salt			
-			request.routed = current_app.get_routed (thing)			
-			request.routable = options
-		
-		current_app.maintern ()	
+			self.env ["wsgi.app"] = current_app
+			self.env ["wsgi.routed"] = current_app.get_routed (thing)
+			self.env ["wsgi.route_options"] = options
+			request.env = self.env
+
+		current_app.maintern ()
 		return current_app, thing, param, respcode
-		
+
 	def __call__ (self):
 		request = self.env ["skitai.was"].request
 		current_app, thing, param, respcode = self.find_method (request, self.env ["PATH_INFO"])
-		
-		if respcode: 
+
+		if respcode:
 			# unacceptable
 			return b""
-		
+
 		current_app.emit ("request:started")
 		self.was = self.env ["skitai.was"]
 		self.was.subapp = current_app
-		
+
 		try:
 			content = self.generate_content (thing, (), param)
+			if html_minify and current_app.config.get ('MINIFY_HTML') and len (content) == 1 and isinstance (content [0], str) and self.was.response.get_header ('content-type') in ('text/html', None):
+				content = [html_minify (content [0])]
+
 		except MemoryError:
 			raise
 		except HTTPError as e:
@@ -258,16 +269,15 @@ class Executor:
 			content = [request.response.with_explain (e.status, e.explain, e.errno)]
 		except:
 			self.was.traceback ()
-			self.rollback ()			
+			self.rollback ()
 			content = [request.response ("502 Bad Gateway", exc_info = self.was.app.debug and sys.exc_info () or None)]
 		else:
 			self.commit ()
-		
+
 		if current_app.debug and current_app.expose_spec and request.channel.addr [0].startswith ("127.0.0.") and isinstance (content [0], API):
 			content [0].set_spec (current_app)
 
 		# clean was
 		del self.was.subapp
-		current_app.emit ("request:finished")		
+		current_app.emit ("request:finished")
 		return content
-		
\ No newline at end of file
